@@ -1,9 +1,10 @@
 /*
- * Copyright 2000-2003 by Hans Reiser, licensing governed by 
+ * Copyright 2000-2004 by Hans Reiser, licensing governed by 
  * reiserfsprogs/README
  */
 
 #include "debugreiserfs.h"
+#include <time.h>
 
 extern struct reiserfs_fsstat g_stat_info;
 
@@ -688,7 +689,7 @@ void do_one_blkh_random_corrupt (struct buffer_head * bh)
 	struct block_head * blkh;
 	unsigned int from;
 	unsigned int count;
-	int i;
+	unsigned int i;
 
 	from = get_rand (0, BLKH_SIZE - 1);
 	count = get_rand (1, BLKH_SIZE);
@@ -780,7 +781,7 @@ void do_one_block_random_corrupt (struct buffer_head * bh)
 {
 	unsigned int from = get_rand (0, bh->b_size - 1);
 	unsigned int count = get_rand (1, bh->b_size);
-	int i;
+	unsigned int i;
 
 	if (from + count > bh->b_size)
 		count = bh->b_size - from;
@@ -801,11 +802,11 @@ void do_one_block_random_corrupt (struct buffer_head * bh)
 void do_bitmap_corruption (reiserfs_filsys_t * fs)
 {
 
-    unsigned long first = fs->fs_super_bh->b_blocknr + 1;
+	unsigned long first = fs->fs_super_bh->b_blocknr + 1;
 	unsigned long nr_bitmap_to_corrupt;
 	unsigned long block;
-    struct buffer_head * bh;
-	int i;
+	struct buffer_head * bh;
+	unsigned int i;
 
 	nr_bitmap_to_corrupt = (unsigned long)get_rand (1, get_sb_bmap_nr (fs->fs_ondisk_sb) - 1);
 
@@ -837,7 +838,7 @@ void do_ih_random_corrupt (reiserfs_filsys_t * fs,
 							  unsigned long nr_leaves_cr) 
 {
 	unsigned int nr_ih_cr;
-	int i, j;
+	unsigned int i, j;
 	struct buffer_head * bh;
 	struct item_head * ih;
 	unsigned long nr_leaves = 0;
@@ -944,7 +945,7 @@ void do_dir_random_corrupt (reiserfs_filsys_t * fs,
 							unsigned long nr_leaves_cr) 
 {
  	unsigned int nr_ih_cr;
-	int i, j;
+	unsigned int i, j;
 	struct buffer_head * bh;
 	struct item_head * ih;
 	unsigned long nr_leaves = 0;
@@ -1025,7 +1026,7 @@ void do_sd_random_corrupt (reiserfs_filsys_t * fs,
 						   unsigned long nr_leaves_cr) 
 {
  	unsigned int nr_ih_cr;
-	int i, j;
+	unsigned int i, j;
 	struct buffer_head * bh;
 	struct item_head * ih;
 	unsigned long nr_leaves = 0;
@@ -1106,7 +1107,7 @@ void do_ind_random_corrupt (reiserfs_filsys_t * fs,
 						   unsigned long nr_leaves_cr) 
 {
  	unsigned int nr_ih_cr;
-	int i, j;
+	unsigned int i, j;
 	struct buffer_head * bh;
 	struct item_head * ih;
 	unsigned long nr_leaves = 0;
