@@ -1,9 +1,11 @@
 /*
- * Copyright 1999-2003 Hans Reiser
+ * Copyright 1999-2004 by Hans Reiser, licensing governed by 
+ * reiserfsprogs/README
  */
 
 #include "fsck.h"
 
+#if 0
 struct check_relocated {
     __u32 old_dir_id;
     __u32 old_objectid;
@@ -81,6 +83,8 @@ void clear_relocated_list() {
     }
 }
 
+#endif
+
 //
 //
 //
@@ -135,22 +139,24 @@ static int did_we_meet_it (unsigned long block)
 
 static void init_control_bitmap (reiserfs_filsys_t * fs)
 {
-    int i;
+    unsigned int i;
     unsigned long block;
     unsigned long reserved;
     
 
-    control_bitmap = reiserfs_create_bitmap (get_sb_block_count (fs->fs_ondisk_sb));
+    control_bitmap = reiserfs_create_bitmap(get_sb_block_count(fs->fs_ondisk_sb));
     if (!control_bitmap)
 	die ("init_control_bitmap: Failed to allocate a control bitmap.");
 
-    /*printf ("Initially number of zeros in control bitmap %d\n", reiserfs_bitmap_zeros (control_bitmap));*/
+    /*printf ("Initially number of zeros in control bitmap %d\n", 
+	      reiserfs_bitmap_zeros (control_bitmap));*/
 
     /* skipped and super block */
     for (i = 0; i <= fs->fs_super_bh->b_blocknr; i ++)
     	we_met_it (i);
 
-    /*printf ("SKIPPED: %d blocks marked used (%d)\n", hits, reiserfs_bitmap_zeros (control_bitmap));
+    /*printf ("SKIPPED: %d blocks marked used (%d)\n", hits, 
+              reiserfs_bitmap_zeros (control_bitmap));
       hits = 0;*/
 
     /* bitmaps */
@@ -159,11 +165,15 @@ static void init_control_bitmap (reiserfs_filsys_t * fs)
         we_met_it (block);
 
 	if (spread_bitmaps (fs))
-	    block = (block / (fs->fs_blocksize * 8) + 1) * (fs->fs_blocksize * 8);
+	    block = (block / (fs->fs_blocksize * 8) + 1) * 
+		    (fs->fs_blocksize * 8);
 	else
 	    block ++;	
     }
-    /*printf ("BITMAPS: %d blocks marked used (%d)\n", hits, reiserfs_bitmap_zeros (control_bitmap));
+    
+    /*printf ("BITMAPS: %d blocks marked used (%d)\n", hits, 
+	      reiserfs_bitmap_zeros (control_bitmap));
+	      
       hits = 0;*/
     
     /* mark as used area of the main device either containing a journal or
@@ -175,7 +185,8 @@ static void init_control_bitmap (reiserfs_filsys_t * fs)
 
     if (!is_new_sb_location (fs->fs_super_bh->b_blocknr, fs->fs_blocksize) &&
     	!is_old_sb_location (fs->fs_super_bh->b_blocknr, fs->fs_blocksize))
-	die ("init_control_bitmap: Wrong super block location. You must run --rebuild-sb.");
+	die ("init_control_bitmap: Wrong super block location. You must run "
+	     "--rebuild-sb.");
 
     block = get_journal_start_must (fs);
 
@@ -198,16 +209,20 @@ static void handle_bitmaps (reiserfs_filsys_t * fs)
     int problem = 0;
 
     if (tree_scanning_failed) {
-	fsck_progress ("Could not scan the internal tree. --rebuild-tree is required\n");
+	fsck_progress ("Could not scan the internal tree. --rebuild-tree "
+		       "is required\n");
 	return;
     }
 
     fsck_progress ("Comparing bitmaps..");
 
     /* check free block counter */
-    if (get_sb_free_blocks (fs->fs_ondisk_sb) != reiserfs_bitmap_zeros (control_bitmap)) {
-/*	fsck_log ("vpf-10630: The count of free blocks in the on-disk bitmap (%lu) mismatches "
-	    "with the correct one (%lu).\n",  get_sb_free_blocks (fs->fs_ondisk_sb),
+    if (get_sb_free_blocks (fs->fs_ondisk_sb) != 
+	reiserfs_bitmap_zeros (control_bitmap)) 
+    {
+/*	fsck_log ("vpf-10630: The count of free blocks in the on-disk bitmap "
+		  "(%lu) mismatches with the correct one (%lu).\n",  
+		  get_sb_free_blocks (fs->fs_ondisk_sb),
 		  reiserfs_bitmap_zeros (control_bitmap));
 */
 	problem++;
@@ -221,28 +236,34 @@ static void handle_bitmaps (reiserfs_filsys_t * fs)
 	    fsck_log ("vpf-10630: The on-disk and the correct bitmaps differs. "
 		"Will be fixed later.\n");
 //            fsck_progress ("Trying to fix bitmap ..\n");
-            /* mark blocks as used in source bitmap if they are used in control bitmap */
+            /* mark blocks as used in source bitmap if they are used in 
+	       control bitmap */
             reiserfs_bitmap_disjunction (source_bitmap, control_bitmap);
-            /* change used blocks count accordinly source bitmap,
-               copy bitmap changes to on_disk bitmap */
-            set_sb_free_blocks (fs->fs_ondisk_sb, reiserfs_bitmap_zeros (source_bitmap));
+            /* change used blocks count accordinly source bitmap, copy bitmap 
+	       changes to on_disk bitmap */
+            set_sb_free_blocks (fs->fs_ondisk_sb, 
+				reiserfs_bitmap_zeros(source_bitmap));
             reiserfs_bitmap_copy (fs->fs_bitmap2, source_bitmap);
             mark_buffer_dirty (fs->fs_super_bh);
 /*
             // check again 
-            if ((diff = reiserfs_bitmap_compare (source_bitmap, control_bitmap)) != 0)  {
-                // do not mark them as fatal or fixable because one can live with leaked space.
-                // So this is not a fatal corruption, and fix-fixable cannot fix it 
-                fsck_progress (" bitmaps were not recovered. \n"
-                "\tYou can either run rebuild-tree or live with %d leaked blocks\n", diff);
+            if ((diff = reiserfs_bitmap_compare(source_bitmap, 
+						control_bitmap)) != 0)  
+	    {
+                // do not mark them as fatal or fixable because one can live 
+		// with leaked space. So this is not a fatal corruption, and 
+		// fix-fixable cannot fix it 
+                fsck_progress (" bitmaps were not recovered.\n\tYou can either "
+		"run rebuild-tree or live with %d leaked blocks\n", diff);
             } else {
                 fsck_progress ("finished\n");
             }
 */
         } else if (problem) {
-	    fsck_log ("vpf-10640: The on-disk and the correct bitmaps differs.\n");	    
+	    fsck_log ("vpf-10640: The on-disk and the correct bitmaps differs.\n");
             while (problem) {
-                /* fixable corruptions because we can try to recover them without rebuilding the tree */
+                /* fixable corruptions because we can try to recover them 
+		   without rebuilding the tree */
         	one_more_corruption (fs, FIXABLE); 
         	problem --;
             }
@@ -310,63 +331,20 @@ static int got_already (reiserfs_filsys_t * fs, unsigned long block)
     return 0;
 }
 
-/* 1 if some of fields in the block head of bh look bad */
-static int bad_block_head (reiserfs_filsys_t * fs, struct buffer_head * bh)
-{
-    struct block_head * blkh;
-    int sum_length = 0;
-
-    blkh = B_BLK_HEAD (bh);
-    if (get_blkh_nr_items (blkh) > (bh->b_size - BLKH_SIZE) / IH_SIZE) {
-	fsck_log ("block %lu: The number of items (%u) exceeds the limit.\n",
-		  bh->b_blocknr, get_blkh_nr_items (blkh));
-	one_more_corruption (fs, FATAL);
-	return 1;
-    }
-
-    if (!does_node_look_like_a_leaf (bh->b_data, bh->b_size)) {
-	fsck_log ("block %lu: The block has wrong structure of items.\n", bh->b_blocknr);
-        one_more_corruption (fs, FATAL);
-	return 1;
-    }
-
-    sum_length = (get_blkh_nr_items (blkh) > 0) ?
-	get_ih_location (B_N_PITEM_HEAD (bh, get_blkh_nr_items (blkh) - 1)) : bh->b_size;
-
-    if (get_blkh_free_space (blkh) !=
-	sum_length - BLKH_SIZE - IH_SIZE * get_blkh_nr_items (blkh))
-    {
-	fsck_log ("block %lu: The free space (%lu) is incorrect, should be (%lu)",
-		bh->b_blocknr, get_blkh_free_space (blkh), sum_length - BLKH_SIZE - IH_SIZE * get_blkh_nr_items (blkh) );
-	if (fsck_mode(fs) == FSCK_FIX_FIXABLE) {
-	    set_blkh_free_space (blkh, sum_length - BLKH_SIZE - IH_SIZE * get_blkh_nr_items (blkh));
-	    fsck_log (" - corrected\n");
-	    mark_buffer_dirty (bh);
-	} else {
-	    fsck_log ("\n");
-	    one_more_corruption (fs, FIXABLE);
-	    return 1;
-	}
-    }
-
-    return 0;
-}
-
 /* 1 if it does not look like reasonable stat data */
-static int bad_stat_data (reiserfs_filsys_t * fs, struct buffer_head * bh, struct item_head * ih)
+static int bad_stat_data (reiserfs_filsys_t * fs, 
+			  struct buffer_head * bh, 
+			  struct item_head * ih)
 {
     unsigned long objectid;
-    struct stat_data * sd;
-    __u32 pos;
 //    __u32 links;
 
-
     objectid = get_key_objectid (&ih->ih_key);
     if (!is_objectid_used (fs, objectid)) {
 	/* FIXME: this could be cured right here */
-	fsck_log ("bad_stat_data: The objectid (%lu) is marked free, but used by an object %k\n",
-	    objectid, &ih->ih_key);	
-        
+	fsck_log ("bad_stat_data: The objectid (%lu) is marked free, but used "
+		  "by an object %k\n", objectid, &ih->ih_key);
+
         /* if it is FIX_FIXABLE we flush objectid map at the end
            no way to call one_less_corruption later
         */
@@ -374,18 +352,19 @@ static int bad_stat_data (reiserfs_filsys_t * fs, struct buffer_head * bh, struc
 	    one_more_corruption (fs, FIXABLE);
     }
 
-    if (is_objectid_really_used (proper_id_map (fs), objectid, &pos)) {
-	fsck_log ("bad_stat_data: The objectid (%lu) is shared by at least two files\n", objectid);
-	if (fsck_mode (fs) == FSCK_FIX_FIXABLE) 
-            to_be_relocated (&ih->ih_key);
-	    
+    if (id_map_mark(proper_id_map (fs), objectid)) {
+	fsck_log ("bad_stat_data: The objectid (%lu) is shared by at least two "
+		  "files. Can be fixed with --rebuild-tree only.\n", objectid);
+#if 0
+	to_be_relocated (&ih->ih_key);
 //	one_more_corruption (fs, FIXABLE);	
-	
-	return 0;
+#endif
     }
 
-    sd = (struct stat_data *)B_I_PITEM(bh,ih);
+    return 0;
 /* Check this on semantic_check pass.    
+
+    sd = (struct stat_data *)B_I_PITEM(bh,ih);
     get_sd_nlink (ih, sd, &links);
     if (S_ISDIR(sd->sd_mode)) {
         if (links < 2) {
@@ -401,19 +380,22 @@ static int bad_stat_data (reiserfs_filsys_t * fs, struct buffer_head * bh, struc
         }
     }
 */    
-    __mark_objectid_really_used (proper_id_map (fs), objectid, pos);
-    return 0;
 }
 
 
 /* it looks like we can check item length only */
-static int bad_direct_item (reiserfs_filsys_t * fs, struct buffer_head * bh, struct item_head * ih)
+static int bad_direct_item (reiserfs_filsys_t * fs, 
+			    struct buffer_head * bh, 
+			    struct item_head * ih)
 {
     return 0;
 }
 
 
-inline void handle_one_pointer (reiserfs_filsys_t * fs, struct buffer_head * bh, __u32 * ptr) {
+inline void handle_one_pointer (reiserfs_filsys_t * fs, 
+				struct buffer_head * bh,
+				__u32 * ptr) 
+{
     if (fsck_mode (fs) == FSCK_FIX_FIXABLE) {
 	fsck_log (" - zeroed");
 	*ptr = 0;
@@ -423,31 +405,38 @@ inline void handle_one_pointer (reiserfs_filsys_t * fs, struct buffer_head * bh,
     }
 }
 
-/*
+
 static int bad_badblocks_item (reiserfs_filsys_t * fs, struct buffer_head * bh,
 			      struct item_head * ih) {
-    int i;
+    __u32 i;
     __u32 * ind = (__u32 *)B_I_PITEM (bh, ih);
 
     if (get_ih_item_len (ih) % 4) {
 	fsck_log ("%s: block %lu: item (%H) has bad length\n", __FUNCTION__,
 		  bh->b_blocknr, ih);
-	one_more_corruption (fs, fatal);
+	one_more_corruption (fs, FATAL);
 	return 1;
     }
-
+	
+    /* All valid badblocks are given in badblock bitmap. Nothing to check anymore. */
+    if (fs->fs_badblocks_bm)
+	    return 0;
+    
     for (i = 0; i < I_UNFM_NUM (ih); i ++) {
-//	__u32 unfm_ptr;
-
-//	unfm_ptr = le32_to_cpu (ind [i]);
 	if (!le32_to_cpu (ind [i])) {
-	    fsck_log ("%s: block %lu: badblocks item (%H) has zero pointer.");
-	    fsck_log ("Not an error, but could be deleted with --fix-fixable\n",
-	    		__FUNCTION__, bh->b_blocknr, ih);
+/*	    fsck_log ("%s: block %lu: badblocks item (%H) has zero pointer.",
+		      __FUNCTION__, bh->b_blocknr, ih);
+	    
+	    if (fsck_mode(fs) != FSCK_FIX_FIXABLE) {
+		    fsck_log("Not an error, but could be deleted with --fix-fixable\n");
+	    } else {
+		    fsck_log("Will be deleted later.\n");
+	    }*/
+	    
 	    continue;
 	}
 
-	// check list of badblocks pointers
+	/* check list of badblocks pointers */
 	if (le32_to_cpu (ind [i]) >= get_sb_block_count (fs->fs_ondisk_sb)) {
 	    fsck_log ("%s: badblock pointer (block %lu) points out of disk spase (%lu)",
 			__FUNCTION__, bh->b_blocknr, le32_to_cpu (ind [i]));
@@ -456,40 +445,42 @@ static int bad_badblocks_item (reiserfs_filsys_t * fs, struct buffer_head * bh,
 	}
 
 	if (did_we_meet_it (le32_to_cpu (ind [i]))) {
-	    // it can be
-	    // 1. not_data_block
-	    // 		delete pointer
-	    // 2. ind [i] or internal/leaf	       		
-	    //    advice to run fix-fixable if there is no fatal errors
-	    //    with list of badblocks, say that it could fix it.
+	    /* it can be
+	       1. not_data_block
+	       		delete pointer
+	       2. ind [i] or internal/leaf
+	          advice to run fix-fixable if there is no fatal errors
+	          with list of badblocks, say that it could fix it. */
 	
 	    if (not_data_block (fs, le32_to_cpu (ind [i]))) {
 		fsck_log ("%s: badblock pointer (block %lu) points on fs metadata (%lu)",
-			__FUNCTION__, bh->b_blocknr, le32_to_cpu (ind [i]));
+			  __FUNCTION__, bh->b_blocknr, le32_to_cpu (ind [i]));
 		handle_one_pointer (fs, bh, &ind[i]);
 		fsck_log ("\n");
 	    } else {
-	        one_more_corruption (fs, badblocks);
-	        fsck_log ("%s: badblock item points to a block"
-	        	  " which in the tree already. Use --badblock-file option"
-	        	  " to fix the problem\n", __FUNCTION__);
+		one_more_corruption(fs, FIXABLE);
+	        fsck_log ("%s: badblock pointer (block %lu) points to a block (%lu) "
+			  "which is in the tree already. Use badblock option (-B) to"
+			  " fix the problem\n", __FUNCTION__, bh->b_blocknr, 
+			  le32_to_cpu(ind[i]));
 	    }
-	} else {
-	    we_met_it (le32_to_cpu (ind [i]));
+
+	    continue;
 	}
+	
+	we_met_it(le32_to_cpu(ind[i]));
     }
 
     return 0;
 }
-*/
 
 /* for each unformatted node pointer: make sure it points to data area and
    that it is not in the tree yet */
 static int bad_indirect_item (reiserfs_filsys_t * fs, struct buffer_head * bh,
 			      struct item_head * ih)
 {
-    int i;
     __u32 * ind = (__u32 *)B_I_PITEM (bh, ih);
+    unsigned int i;
 
     if (get_ih_item_len (ih) % 4) {
 	fsck_log ("%s: block %lu: The item (%H) has the bad length (%u)\n",
@@ -511,16 +502,20 @@ static int bad_indirect_item (reiserfs_filsys_t * fs, struct buffer_head * bh,
 	/* check unformatted node pointer and mark it used in the
            control bitmap */
 	if (bad_block_number (fs, le32_to_cpu (ind [i]))) {
-	    fsck_log ("%s: block %lu: The item %k has the bad pointer (%d) to the block (%lu)",
-		      __FUNCTION__, bh->b_blocknr, &ih->ih_key, i, le32_to_cpu (ind [i]));
+	    fsck_log ("%s: block %lu: The item %k has the bad pointer (%d) to "
+		      "the block (%lu)", __FUNCTION__, bh->b_blocknr, 
+		      &ih->ih_key, i, le32_to_cpu (ind [i]));
+	    
 	    handle_one_pointer (fs, bh, &ind[i]);
 	    fsck_log ("\n");
 	    continue;
 	}
 
         if (got_already (fs, le32_to_cpu (ind [i]))) {
-	    fsck_log ("%s: block %lu: The item (%H) has the bad pointer (%d) to the block (%lu), "
-		"which is in tree already", __FUNCTION__, bh->b_blocknr, ih, i, le32_to_cpu (ind [i]));
+	    fsck_log ("%s: block %lu: The item (%H) has the bad pointer (%d) "
+		      "to the block (%lu), which is in tree already", 
+		      __FUNCTION__, bh->b_blocknr, ih, i, le32_to_cpu(ind[i]));
+	    
 	    handle_one_pointer (fs, bh, &ind [i]);
 	    fsck_log ("\n");
             continue;
@@ -544,11 +539,13 @@ static int bad_indirect_item (reiserfs_filsys_t * fs, struct buffer_head * bh,
 
 
 /* FIXME: this was is_bad_directory from pass0.c */
-static int bad_directory_item (reiserfs_filsys_t * fs, struct buffer_head * bh, struct item_head * ih)
+static int bad_directory_item (reiserfs_filsys_t * fs, 
+			       struct buffer_head * bh, 
+			       struct item_head * ih)
 {
-    int i;
     char * name;
     int namelen;
+    unsigned int count, i;
     struct reiserfs_de_head * deh = B_I_DEH (bh, ih);
     int min_entry_size = 1;/* We have no way to understand whether the
                               filesystem was created in 3.6 format or
@@ -556,30 +553,43 @@ static int bad_directory_item (reiserfs_filsys_t * fs, struct buffer_head * bh,
                               length is 1 */
     __u16 state;
 
+    count = get_ih_entry_count (ih);
+
+    if (count == 0) {
+	one_more_corruption (fs, FATAL);	
+	return 1;
+    }
+    
     /* make sure item looks like a directory */
-    if (get_ih_item_len (ih) / (DEH_SIZE + min_entry_size) < get_ih_entry_count (ih)) {
+    if (get_ih_item_len (ih) / (DEH_SIZE + min_entry_size) < count) {
 	/* entry count can not be that big */
-	fsck_log ("%s: block %lu: The directory item %k has the exsessively big entry count (%u)\n",
-	    __FUNCTION__, bh->b_blocknr, &ih->ih_key, get_ih_entry_count (ih));
+	fsck_log ("%s: block %lu: The directory item %k has the exsessively "
+		  "big entry count (%u)\n",  __FUNCTION__, bh->b_blocknr, 
+		  &ih->ih_key, count);
+	
 	one_more_corruption (fs, FATAL);	
 	return 1;
     }
 
-    if (get_deh_location (&deh[get_ih_entry_count (ih) - 1]) != DEH_SIZE * get_ih_entry_count (ih)) {
+    if (get_deh_location (&deh[count - 1]) != DEH_SIZE * count) {
 	/* last entry should start right after array of dir entry headers */
-	fsck_log ("%s: block %lu: The directory item %k has the corrupted entry structure\n",
-	    __FUNCTION__, bh->b_blocknr, &ih->ih_key);
-	one_more_corruption (fs, FATAL);	
+	fsck_log ("%s: block %lu: The directory item %k has the corrupted "
+		  "entry structure\n", __FUNCTION__, bh->b_blocknr, 
+		  &ih->ih_key);
+	
+	one_more_corruption (fs, FATAL);
 	return 1;
     }
-
+    
     /* check name hashing */
-    for (i = 0; i < get_ih_entry_count (ih); i ++, deh ++) {
+    for (i = 0; i < count; i ++, deh ++) {
 	namelen = name_in_entry_length (ih, deh, i);
 	name = name_in_entry (deh, i);
 	if (!is_properly_hashed (fs, name, namelen, get_deh_offset (deh))) {
-	    fsck_log ("%s: block %lu: The directory item %k has a not properly hashed entry (%d)\n",
-			__FUNCTION__, bh->b_blocknr, &ih->ih_key, i);
+	    fsck_log ("%s: block %lu: The directory item %k has a not properly "
+		      "hashed entry (%d)\n", __FUNCTION__, bh->b_blocknr, 
+		      &ih->ih_key, i);
+	    
 	    one_more_corruption (fs, FATAL);	
 	    return 1;
 	}
@@ -588,12 +598,14 @@ static int bad_directory_item (reiserfs_filsys_t * fs, struct buffer_head * bh,
     deh = B_I_DEH (bh, ih);
     state = (1 << DEH_Visible2);
     /* ok, items looks like a directory */
-    for (i = 0; i < get_ih_entry_count (ih); i ++, deh ++) {
+    for (i = 0; i < count; i ++, deh ++) {
 	if (get_deh_state (deh) != state) {
-	    fsck_log ("bad_directory_item: block %lu: The directory item %k has the entry (%d) "
-		      "\"%.*s\" with a not legal state (%o), (%o) expected",
-		      bh->b_blocknr, &ih->ih_key, i, name_in_entry_length (ih, deh, i),
-		      name_in_entry (deh, i), get_deh_state (deh), state);
+	    fsck_log ("bad_directory_item: block %lu: The directory item %k "
+		      "has the entry (%d) \"%.*s\" with a not legal state "
+		      "(%o), (%o) expected", bh->b_blocknr, &ih->ih_key, i, 
+		      name_in_entry_length (ih, deh, i), name_in_entry (deh, i), 
+		      get_deh_state (deh), state);
+	    
 	    if (fsck_mode (fs) == FSCK_FIX_FIXABLE) {
 		set_deh_state (deh, 1 << DEH_Visible2);
 		mark_buffer_dirty (bh);
@@ -619,11 +631,12 @@ static int bad_item (reiserfs_filsys_t * fs, struct buffer_head * bh, int num)
     if ((get_ih_flags(ih)) != 0) {
 	if (fsck_mode(fs) != FSCK_FIX_FIXABLE) {
 	    one_more_corruption (fs, FIXABLE);
-	    fsck_log ("%s: vpf-10570: block %lu: The item header (%d) has not cleaned flags.\n", 
-		__FUNCTION__, bh->b_blocknr, num);
+	    fsck_log ("%s: vpf-10570: block %lu: The item header (%d) has not "
+		      "cleaned flags.\n", __FUNCTION__, bh->b_blocknr, num);
 	} else {
-	    fsck_log ("%s: vpf-10580: block %lu: Flags in the item header (%d) were cleaned\n",
-		__FUNCTION__, bh->b_blocknr, num);
+	    fsck_log ("%s: vpf-10580: block %lu: Flags in the item header "
+		      "(%d) were cleaned\n", __FUNCTION__, bh->b_blocknr, num);
+	    
 	    clean_ih_flags(ih);
 	    mark_buffer_dirty(bh);
 	}
@@ -640,72 +653,54 @@ static int bad_item (reiserfs_filsys_t * fs, struct buffer_head * bh, int num)
     if (format != get_ih_key_format(ih)) {
 	if (fsck_mode(fs) != FSCK_FIX_FIXABLE) {
 	    one_more_corruption (fs, FIXABLE);
-	    fsck_log ("%s: vpf-10710: block %lu: The format (%d) specified in the item header (%d) "
-		"differs from the key format (%d).\n", __FUNCTION__, bh->b_blocknr, 
-		get_ih_key_format(ih), num, format);
+	    fsck_log ("%s: vpf-10710: block %lu: The format (%d) specified "
+		      "in the item header (%d) differs from the key format "
+		      "(%d).\n", __FUNCTION__, bh->b_blocknr, 
+		      get_ih_key_format(ih), num, format);
 	} else {
-	    fsck_log ("%s: vpf-10720: block %lu: The format (%d) specified in the item header (%d) "
-		"was fixed to the key format (%d).\n", __FUNCTION__, bh->b_blocknr, 
-		get_ih_key_format(ih), num, format);
+	    fsck_log ("%s: vpf-10720: block %lu: The format (%d) specified in "
+		      "the item header (%d) was fixed to the key format (%d).\n", 
+		      __FUNCTION__, bh->b_blocknr, get_ih_key_format(ih), num, 
+		      format);
 
 	    set_ih_key_format(ih, format);
 	    mark_buffer_dirty(bh);
 	}
     }
 
-    if (get_key_dirid (&ih->ih_key) == (__u32)-1) {
-	if (get_key_objectid (&ih->ih_key) != (__u32)-1) {
-	    /*  safe link can be
+    if (get_key_objectid (&ih->ih_key) == BADBLOCK_OBJID) {
+	if (get_key_dirid (&ih->ih_key) == BADBLOCK_DIRID && is_indirect_ih (ih)) {
+		/* Bad Block support. */
+		return bad_badblocks_item (fs, bh, ih);
+	}
+
+	goto error;
+    } else {
+	if (get_key_dirid (&ih->ih_key) == (__u32)-1) {
+	    /*  Safe Link support. Allowable safe links are:
 		-1 object_id           0x1	INDIRECT (truncate) or
-		-1 object_id   blocksize+1	DIRECT   (unlink)
-	    */
+		-1 object_id   blocksize+1	DIRECT   (unlink) */
+	    if (is_direct_ih(ih) && get_offset(&ih->ih_key) == fs->fs_blocksize + 1) {
+		if (get_ih_item_len (ih) == 4) {
+		    /*fsck_log("vpf-00010: safe link found %k\n", &ih->ih_key);*/
+		    fsck_check_stat(fs)->safe ++;
+		    return 0;
+		}
+	    }
 
-	    if (is_direct_ih(ih))
-		if (get_offset(&ih->ih_key) == fs->fs_blocksize + 1)
-		    if (get_ih_item_len (ih) == 4) {
-			/*fsck_log("vpf-00010: safe link found %k\n", &ih->ih_key);*/
-			fsck_check_stat(fs)->safe ++;
-			return 0;
-		    }
-
-	    if (is_indirect_ih(ih))
-		if(get_offset(&ih->ih_key) == 0x1)
-		    if (get_ih_item_len (ih) == 4) {
-			/*fsck_log("vpf-00020: safe link found %k\n", &ih->ih_key);*/
-			fsck_check_stat(fs)->safe ++;
-			return 0;
-		    }
+	    if (is_indirect_ih(ih) && get_offset(&ih->ih_key) == 0x1) {
+		if (get_ih_item_len (ih) == 4) {
+		    /*fsck_log("vpf-00020: safe link found %k\n", &ih->ih_key);*/
+		    fsck_check_stat(fs)->safe ++;
+		    return 0;
+		}
+	    }
 	
 	    /* it does not look like safe link */
-
-	    /* dir_id == -1 can be used only by safe links */
-	    one_more_corruption (fs, FATAL);
-	    fsck_log ("%s: vpf-10290: block %lu, item %d: The item has a wrong key %k\n", 
-		__FUNCTION__, num, bh->b_blocknr, &ih->ih_key);
-	    return 1;
-	} else {
-
-/*              BAD BLOCK LIST SUPPORT
-            if (is_indirect_ih (ih)) {
-	    // it looks like badblocks item
-	    if (fs->fs_badblocks_bm)
-	    	return 0;
-	    else
-		return bad_badblocks_item (fs, bh, ih);
-	} else  {*/
-
-	    one_more_corruption (fs, FATAL);
-	    fsck_log ("%s: vpf-10300: block %lu, item %d: The item has a wrong key %k\n", 
-		__FUNCTION__, num, bh->b_blocknr, &ih->ih_key);
-	    return 1;
+	    goto error;
 	}
-    } else if (get_key_objectid (&ih->ih_key) == (__u32)-1) {
-	one_more_corruption (fs, FATAL);
-	fsck_log ("%s: vpf-10310: block %lu, item %d: The item has a wrong key %k\n", 
-	    __FUNCTION__, num, bh->b_blocknr, &ih->ih_key);
-	return 1;
     }
-
+    
     if (is_stat_data_ih (ih))
 	return bad_stat_data (fs, bh, ih);
 
@@ -716,12 +711,17 @@ static int bad_item (reiserfs_filsys_t * fs, struct buffer_head * bh, int num)
 	return bad_indirect_item (fs, bh, ih);
     
     return bad_directory_item (fs, bh, ih);
+ 
+ error:
+    one_more_corruption (fs, FATAL);
+    fsck_log ("%s: vpf-10310: block %lu, item %d: The item has a wrong "
+	      "key %k\n", __FUNCTION__, num, bh->b_blocknr, &ih->ih_key);
+    return 1;
 }
 
 
 /* 1 if i-th and (i-1)-th items can not be neighbors in a leaf */
-int bad_pair (reiserfs_filsys_t * fs, struct buffer_head * bh, int pos)
-{
+int bad_pair (reiserfs_filsys_t * fs, struct buffer_head * bh, int pos) {
     struct item_head * ih;
 
     ih = B_N_PITEM_HEAD (bh, pos);
@@ -740,15 +740,15 @@ int bad_pair (reiserfs_filsys_t * fs, struct buffer_head * bh, int pos)
 	    return 1;
 	}
 
-    if (get_key_dirid (&ih->ih_key) == (__u32)-1) {
-    	if (get_key_objectid (&ih->ih_key) == (__u32)-1) {
-/*              BAD BLOCK LIST SUPPORT
-    	    if (is_indirect_ih (ih) && comp_short_keys (&((ih - 1)->ih_key), &ih->ih_key) != 0)
-		return 0; // badblocks item
-*/
-	    /* it does not look like badblocks item */
-    	} else {
-    	    /* safe link ? */
+    if (get_key_objectid (&ih->ih_key) == BADBLOCK_OBJID) {
+	/* BAD BLOCK LIST SUPPORT. */
+	if (get_key_dirid(&ih->ih_key) == BADBLOCK_DIRID && 
+	    is_indirect_ih(ih) && comp_short_keys(&((ih - 1)->ih_key), 
+						  &ih->ih_key))
+		return 0;
+    } else {
+	/* Safe link support. */
+	if (get_key_dirid (&ih->ih_key) == (__u32)-1) {
 	    if (comp_short_keys (&((ih - 1)->ih_key), &ih->ih_key) == 0) {
 		if (is_indirect_ih (ih - 1) && is_direct_ih(ih))
 		    return 0; /* safe link */
@@ -758,24 +758,25 @@ int bad_pair (reiserfs_filsys_t * fs, struct buffer_head * bh, int pos)
 		    return 0; /* safe link */
 		/* it does not look like safe link */
 	    }
-    	}
+	}
     }
-
+    
     if (is_direct_ih(ih)) {
-	/* left item must be indirect or stat data item of the same
-	   file */
+	/* left item must be indirect or stat data item of the same file */
 	if (not_of_one_file (&((ih - 1)->ih_key), &ih->ih_key)) {
 	    if (fsck_mode (fs) != FSCK_REBUILD)
 		one_more_corruption (fs, FATAL);
+	    
 	    return 1;
 	}
 
 	if (!((is_stat_data_ih (ih - 1) && get_offset (&ih->ih_key) == 1) ||
-	      (is_indirect_ih (ih - 1) &&	
-	       get_offset (&(ih - 1)->ih_key) + get_bytes_number (ih-1, bh->b_size) ==
-	       get_offset (&ih->ih_key)))) {
+	      (is_indirect_ih (ih - 1) && get_offset (&(ih - 1)->ih_key) + 
+	       get_bytes_number (ih-1, bh->b_size) == get_offset (&ih->ih_key)))) 
+	{
 	    if (fsck_mode (fs) != FSCK_REBUILD)
 		one_more_corruption (fs, FATAL);
+	    
 	    return 1;
 	}
     }
@@ -798,19 +799,20 @@ static int bad_leaf (reiserfs_filsys_t * fs, struct buffer_head * bh)
 {
     int i;
 
-    if (bad_block_head (fs, bh)) {
+    if (leaf_structure_check(fs, bh))
 	return 1;
-    }
     
     for (i = 0; i < B_NR_ITEMS (bh); i ++) {
 	if (bad_item (fs, bh, i)) {
-	    fsck_log ("bad_leaf: block %lu, item %d: The corrupted item found (%H)\n",
-		bh->b_blocknr, i, B_N_PITEM_HEAD (bh, i));
+	    fsck_log ("bad_leaf: block %lu, item %d: The corrupted item found "
+		      "(%H)\n", bh->b_blocknr, i, B_N_PITEM_HEAD (bh, i));
 	}
 
 	if (i && bad_pair (fs, bh, i)) {
-	    fsck_log ("bad_leaf: block %lu, items %d and %d: The wrong order of items: %k, %k\n",
-		bh->b_blocknr, i-1, i, &B_N_PITEM_HEAD (bh, i-1)->ih_key, &B_N_PITEM_HEAD (bh, i)->ih_key);
+	    fsck_log ("bad_leaf: block %lu, items %d and %d: The wrong order "
+		      "of items: %k, %k\n", bh->b_blocknr, i - 1, i, 
+		      &B_N_PITEM_HEAD (bh, i-1)->ih_key, 
+		      &B_N_PITEM_HEAD (bh, i)->ih_key);
 	}
     }
     return 0;
@@ -824,17 +826,24 @@ static int bad_internal (reiserfs_filsys_t * fs, struct buffer_head * bh)
     for (i = 0; i <= B_NR_ITEMS (bh); i ++) {
         if (i != B_NR_ITEMS (bh) && i != B_NR_ITEMS (bh) - 1)
 	    /* make sure that keys are in increasing order */
-            if (comp_keys (B_N_PDELIM_KEY (bh, i), B_N_PDELIM_KEY (bh, i + 1)) != -1) {
-		fsck_log ("%s: vpf-10320: block %lu, items %d and %d: The wrong order of items: %k, %k\n",
-		    __FUNCTION__, bh->b_blocknr, i, i+1, B_N_PDELIM_KEY (bh, i), B_N_PDELIM_KEY (bh, i + 1));
+            if (comp_keys (B_N_PDELIM_KEY (bh, i), 
+			   B_N_PDELIM_KEY (bh, i + 1)) != -1) 
+	    {
+		fsck_log ("%s: vpf-10320: block %lu, items %d and %d: The "
+			  "wrong order of items: %k, %k\n", __FUNCTION__, 
+			  bh->b_blocknr, i, i + 1, B_N_PDELIM_KEY (bh, i), 
+			  B_N_PDELIM_KEY (bh, i + 1));
+		
 		one_more_corruption (fs, FATAL);
                 return 1;
 	    }
 
 	/* make sure that the child is correct */
         if (bad_block_number (fs, get_dc_child_blocknr (B_N_CHILD (bh,i)))) {
-	    fsck_log ("%s: vpf-10330: block %lu, item %d: The internal item points to the not legal block (%lu)\n",
-		__FUNCTION__, bh->b_blocknr, i, get_dc_child_blocknr (B_N_CHILD (bh,i)));
+	    fsck_log ("%s: vpf-10330: block %lu, item %d: The internal item "
+		      "points to the not legal block (%lu)\n", __FUNCTION__, 
+		      bh->b_blocknr, i, get_dc_child_blocknr (B_N_CHILD (bh,i)));
+	    
             one_more_corruption (fs, FATAL);
             return 1;
 	}
@@ -849,6 +858,72 @@ static inline int h_to_level (reiserfs_filsys_t * fs, int h)
     return get_sb_tree_height (fs->fs_ondisk_sb) - h - 1;
 }
 
+#if 0
+int dc_fix(struct buffer_head *bh, int pos, __u32 block) {
+	if (!bh || !bh->b_data)
+		return -1;
+
+	if (B_NR_ITEMS(bh) < pos)
+		return -1;
+
+	set_dc_child_blocknr(B_N_CHILD(bh,pos), block);
+
+	mark_buffer_dirty(bh);
+	bwrite(bh);
+
+	return 0;
+}
+
+/* Removes @N-th key and @(N+1) pointer. */
+int internal_remove(struct buffer_head *bh, int pos) {
+	char *delete;
+	__u32 nr;
+	
+	if (!bh || !bh->b_data)
+		return -1;
+
+	if (B_NR_ITEMS(bh) < pos)
+		return -1;
+
+	delete = (char *)B_N_CHILD(bh, pos + 2);
+	memmove(delete - DC_SIZE, delete, 
+		bh->b_size - (delete - bh->b_data));
+
+	delete = (char *)B_N_PDELIM_KEY(bh, pos + 1);
+	memmove(delete - KEY_SIZE, delete, 
+		bh->b_size - (delete - bh->b_data));
+
+	nr = B_NR_ITEMS(bh) - 1;
+	
+	set_blkh_nr_items(B_BLK_HEAD(bh), nr);
+	set_blkh_free_space(B_BLK_HEAD(bh), bh->b_size - 
+			    (BLKH_SIZE + KEY_SIZE * nr + DC_SIZE * (nr + 1)));
+	
+	mark_buffer_dirty(bh);
+	bwrite(bh);
+	
+	return 0;
+}
+
+int leaf_fix_key_oid(struct buffer_head *bh, int pos, __u32 oid) {
+	struct item_head * ih;
+
+
+	if (!bh || !bh->b_data)
+		return -1;
+
+	if (B_NR_ITEMS(bh) < pos)
+		return -1;
+
+	ih = B_N_PITEM_HEAD (bh, pos);
+	set_key_objectid(&ih->ih_key, oid);
+	
+	mark_buffer_dirty(bh);
+	bwrite(bh);
+
+	return 0;
+}
+#endif
 
 /* bh must be formatted node. blk_level must be tree_height - h + 1 */
 static int bad_node (reiserfs_filsys_t * fs, struct buffer_head ** path, int h)
@@ -856,22 +931,24 @@ static int bad_node (reiserfs_filsys_t * fs, struct buffer_head ** path, int h)
     struct buffer_head ** pbh = &path[h];
 
     if (B_LEVEL (*pbh) != h_to_level (fs, h)) {
-       	fsck_log ("block %lu: The level of the node (%d) is not correct, (%d) expected\n",
-	    (*pbh)->b_blocknr, B_LEVEL (*pbh), h_to_level (fs, h));
+       	fsck_log ("block %lu: The level of the node (%d) is not correct, "
+		  "(%d) expected\n", (*pbh)->b_blocknr, B_LEVEL (*pbh), 
+		  h_to_level (fs, h));
+	
 	one_more_corruption (fs, FATAL);
         return 1;
     }
 
     if (bad_block_number (fs, (*pbh)->b_blocknr)) {
 	one_more_corruption (fs, FATAL);
-	fsck_log ("%s: vpf-10340: The node in the wrong block number (%lu) found in the tree\n",
-	      __FUNCTION__, (*pbh)->b_blocknr);
+	fsck_log ("%s: vpf-10340: The node in the wrong block number (%lu) "
+		  "found in the tree\n",  __FUNCTION__, (*pbh)->b_blocknr);
 	return 1;
     }
 
     if (got_already (fs, (*pbh)->b_blocknr)) {
-	fsck_log ("%s: vpf-10350: The block (%lu) is used more then once in the tree.\n",
-	      __FUNCTION__, (*pbh)->b_blocknr);
+	fsck_log ("%s: vpf-10350: The block (%lu) is used more than once "
+		  "in the tree.\n",  __FUNCTION__, (*pbh)->b_blocknr);
 	one_more_corruption (fs, FATAL);
         return 1;
     }
@@ -954,8 +1031,9 @@ static int bad_path (reiserfs_filsys_t * fs, struct buffer_head ** path, int h1)
     if (dk && comp_keys (dk, B_N_PKEY (path[h], 0))) {
 	/* left delimiting key must be equal to the key of 0-th item in the
 	   node */
-	fsck_log ("bad_path: The left delimiting key %k of the node (%lu) must be equal to the first element's "
-	    "key %k within the node.\n", dk, path[h]->b_blocknr, B_N_PKEY (path[h], 0));
+	fsck_log ("bad_path: The left delimiting key %k of the node (%lu) must "
+	    "be equal to the first element's key %k within the node.\n", dk, 
+	    path[h]->b_blocknr, B_N_PKEY (path[h], 0));
 	one_more_corruption (fs, FATAL);
 	return 1;
     }
@@ -963,9 +1041,10 @@ static int bad_path (reiserfs_filsys_t * fs, struct buffer_head ** path, int h1)
     dk = rkey (path, h);
     if (dk && comp_keys (dk, B_N_PKEY (path[h],
        get_blkh_nr_items (B_BLK_HEAD (path[h])) - 1)) != 1) {
-	/* right delimiting key must be greater than the key of the last item in the node */
-	fsck_log ("bad_path: The right delimiting key %k of the node (%lu) must be greater then the last (%d) element's"
-	    "key %k within the node.\n", dk, path[h]->b_blocknr, get_blkh_nr_items (B_BLK_HEAD (path[h])) - 1, 
+	/* right delimiting key must be gt the key of the last item in the node */
+	fsck_log ("bad_path: The right delimiting key %k of the node (%lu) must "
+	    "be greater than the last (%d) element's key %k within the node.\n", 
+	    dk, path[h]->b_blocknr, get_blkh_nr_items (B_BLK_HEAD (path[h])) - 1, 
 	    B_N_PKEY (path[h], get_blkh_nr_items (B_BLK_HEAD (path[h])) - 1));
 	one_more_corruption (fs, FATAL);
 	return 1;
@@ -976,15 +1055,20 @@ static int bad_path (reiserfs_filsys_t * fs, struct buffer_head ** path, int h1)
     	BLKH_SIZE != path[h]->b_size))
     {
 	/* wrong dc_size */
-	fsck_log ("bad_path: block %lu, pointer %d: The used space (%d) of the child block (%lu)",
-	    path[h-1]->b_blocknr, pos, get_dc_child_size (B_N_CHILD(path[h-1],pos)), path[h]->b_blocknr);
-	fsck_log (" is not equal to the (blocksize (4096) - free space (%d) - header size (%u))",
-		get_blkh_free_space ((struct block_head *)path[h]->b_data), BLKH_SIZE);
+	fsck_log ("bad_path: block %lu, pointer %d: The used space (%d) of the "
+	    "child block (%lu)", path[h-1]->b_blocknr, pos, 
+	    get_dc_child_size(B_N_CHILD(path[h-1],pos)), path[h]->b_blocknr);
+	fsck_log (" is not equal to the (blocksize (4096) - free space (%d) - "
+	    "header size (%u))", 
+	    get_blkh_free_space((struct block_head *)path[h]->b_data), BLKH_SIZE);
 	
 	if (fsck_mode (fs) == FSCK_FIX_FIXABLE) {
 	    set_dc_child_size (B_N_CHILD(path[h-1],pos), path[h]->b_size -
-	    		get_blkh_free_space ((struct block_head *)path[h]->b_data) - BLKH_SIZE);
-	    fsck_log (" - corrected to (%lu)\n", get_dc_child_size (B_N_CHILD(path[h-1],pos)));
+			       get_blkh_free_space ((struct block_head *)path[h]->b_data) 
+			       - BLKH_SIZE);
+	    fsck_log (" - corrected to (%lu)\n", 
+		      get_dc_child_size (B_N_CHILD(path[h-1], pos)));
+	    
 	    mark_buffer_dirty (path[h-1]);
 	} else {
 	    one_more_corruption (fs, FIXABLE);
@@ -1002,7 +1086,7 @@ static void before_check_fs_tree (reiserfs_filsys_t * fs) {
     source_bitmap = reiserfs_create_bitmap (get_sb_block_count (fs->fs_ondisk_sb));
     reiserfs_bitmap_copy (source_bitmap, fs->fs_bitmap2);
 
-    proper_id_map (fs) = init_id_map (MAP_NOT_PACKED);
+    proper_id_map (fs) = id_map_init();
 }
 
 static void after_check_fs_tree (reiserfs_filsys_t * fs) {
@@ -1036,20 +1120,24 @@ void check_fs_tree (reiserfs_filsys_t * fs)
 	    fprintf(stderr, "The on-disk bitmap looks corrupted.");
 	    one_more_corruption(fs, FIXABLE);
 	}
-	free_id_map(proper_id_map(fs));
+	id_map_free(proper_id_map(fs));
     } else
 	handle_bitmaps (fs);
     
     after_check_fs_tree (fs);
 }
 
-static int clean_attributes_handler (reiserfs_filsys_t * fs, struct buffer_head ** path, int h) {
+static int clean_attributes_handler (reiserfs_filsys_t * fs, 
+				     struct buffer_head ** path, 
+				     int h) 
+{
     struct buffer_head * bh = path[h];
     int i;
 
     if (B_LEVEL (bh) != h_to_level (fs, h)) {
-       	reiserfs_panic ("The node (%lu) with wrong level (%d) found in the tree, (%d) expected\n",
-		  bh->b_blocknr, B_LEVEL (bh), h_to_level (fs, h));
+       	reiserfs_panic ("The node (%lu) with wrong level (%d) found in the "
+			"tree, (%d) expected\n", bh->b_blocknr, B_LEVEL (bh), 
+			h_to_level (fs, h));
     }
 
     if (!is_leaf_node (bh))
@@ -1071,5 +1159,3 @@ void do_clean_attributes (reiserfs_filsys_t * fs) {
     set_sb_v2_flag (fs->fs_ondisk_sb, reiserfs_attrs_cleared);
     mark_buffer_dirty (fs->fs_super_bh);
 }
-
-
