@@ -5,24 +5,19 @@
 
 #define _GNU_SOURCE
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <asm/types.h>
-#include <sys/vfs.h>
-#include <errno.h>
-#include <time.h>
-#include <asm/types.h>
-#include <assert.h>
+#ifndef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include "io.h"
 #include "misc.h"
 #include "reiserfs_lib.h"
 
-#include "../include/config.h"
+#include <string.h>
+#include <errno.h>
+#include <time.h>
+#include <assert.h>
+
 #include "../version.h"
 
 /* main.c */
@@ -210,9 +205,11 @@ void do_clean_attributes (reiserfs_filsys_t * fs);
 int bad_pair (reiserfs_filsys_t *, struct buffer_head * bh, int i);
 int bad_leaf_2 (reiserfs_filsys_t *, struct buffer_head * bh);
 
+#if 0
 extern int should_be_relocated (struct key * key);
 extern void to_be_relocated (struct key * key);
 extern void clear_relocated_list(void);
+#endif
 
 /* ustree.c */
 void reiserfsck_paste_into_item (struct path * path, const char * body, int size);
@@ -250,7 +247,6 @@ int is_block_uninsertable (unsigned long block);
 
 /* objectid.c */
 int comp_ids(const void *p1, const void *p2);
-int is_objectid_used (reiserfs_filsys_t *, __u32 objectid);
 
 typedef struct id_map {
     void **index;
@@ -408,6 +404,7 @@ struct check_info {
 struct fsck_data {
     unsigned short mode; /* check, rebuild, etc*/
     unsigned long options;
+    unsigned long mounted;
 
     struct rebuild_info rebuild;
     struct check_info check;
