@@ -1,10 +1,9 @@
 /*
- * Copyright 1996-2003 by Hans Reiser, licensing governed by 
+ * Copyright 1996-2004 by Hans Reiser, licensing governed by 
  * reiserfsprogs/README 
  */
 
 #include "fsck.h"
-#include <stdlib.h>
 
 reiserfs_bitmap_t * bad_unfm_in_tree_once_bitmap;
 
@@ -544,8 +543,9 @@ static void find_allocable_blocks (reiserfs_filsys_t * fs)
 	}
 
 	/* make allocable not leaves, not bad blocks */
-	if (!is_used_leaf (i) &&
-	    (!fs->fs_badblocks_bm || !reiserfs_bitmap_test_bit (fs->fs_badblocks_bm, i))) {
+	if (!is_used_leaf (i) && (!fs->fs_badblocks_bm || 
+				  !reiserfs_bitmap_test_bit (fs->fs_badblocks_bm, i))) 
+	{
 	    /* this is not leaf and it is not pointed by found indirect items,
                so it does not contains anything valuable */
 	    make_allocable (i);
