@@ -1,47 +1,35 @@
 /*
- * Copyright 1996-2003 Hans Reiser, see reiserfs/README for licensing and copyright details
+ * Copyright 1996-2004 by Hans Reiser, licensing governed by 
+ * reiserfsprogs/README
  */
-#define _GNU_SOURCE
-
-#include <stdio.h>
-#include <unistd.h>
-#include <string.h>
-#include <errno.h>
-#include <stdlib.h> 
-#include <asm/types.h>
-
-#include <sys/types.h>
-#include <sys/stat.h>
-
 
 #include "io.h"
-#include "misc.h"
-#include "config.h"
+
+#include <string.h>
+#include <errno.h>
+#include <asm/types.h>
 
 void check_memory_msg (void) {
     fprintf(stderr, 
-	"\nThe problem has occurred looks like a hardware problem (perhaps memory).\n"
-        "Send us the bug report only if the second run dies at the same place with\n"
-        "the same block number.\n");
+	"\nThe problem has occurred looks like a hardware problem (perhaps\n"
+        "memory). Send us the bug report only if the second run dies at\n"
+	"the same place with the same block number.\n");
 }
 
 void check_hd_msg (void) {
     fprintf(stderr, 
-	"\nThe problem has occurred looks like a hardware problem.\n"
-	"If you have bad blocks, we advise you to get a new hard\n"
-	"drive, because once you get one bad block that the disk\n"
-        "drive internals cannot hide from your sight, the chances\n"
-        "of getting more are generally said to become much higher\n"
-        "(precise statistics are unknown to us), and this disk drive\n"
-        "is probably not expensive enough for you to risk your time\n"
-        "and data on it. If you don't want to follow that advice,\n"
-        "then if you have just a few bad blocks, try writing to the\n"
-        "bad blocks and see if the drive remaps the bad blocks (that\n"
-        "means it takes a block it has in reserve and allocates it\n"
-        "for use for requests of that block number).  If it cannot\n"
-        "remap the block, this could be quite bad, as it may mean\n"
-        "that so many blocks have gone bad that none remain in \n"
-	"reserve to allocate.\n");
+	"\nThe problem has occurred looks like a hardware problem. If you have\n"
+	"bad blocks, we advise you to get a new hard drive, because once you\n"
+	"get one bad block  that the disk  drive internals  cannot hide from\n"
+        "your sight,the chances of getting more are generally said to become\n"
+        "much higher  (precise statistics are unknown to us), and  this disk\n"
+        "drive is probably not expensive enough  for you to you to risk your\n"
+        "time and  data on it.  If you don't want to follow that follow that\n"
+        "advice then  if you have just a few bad blocks,  try writing to the\n"
+	"bad blocks  and see if the drive remaps  the bad blocks (that means\n"
+	"it takes a block  it has  in reserve  and allocates  it for use for\n"
+	"of that block number).  If it cannot remap the block,  use badblock\n"
+	"option (-B) with  reiserfs utils to handle this block correctly.\n");
 }
 
 static int is_bad_block (unsigned long block)
@@ -131,8 +119,8 @@ static void _show_buffers(struct buffer_head **list, int dev, unsigned long size
 	    break;
     }
 
-    printf ("show_buffers (dev %d, size %lu): free %d, count != 0 %d, dirty %d, all %d\n",
-	dev, size, free, in_use, dirty, all);
+    printf("show_buffers (dev %d, size %lu): free %d, count != 0 %d, dirty %d, "
+	"all %d\n", dev, size, free, in_use, dirty, all);
 }
 
 
@@ -300,7 +288,7 @@ static int grow_buffers (int size)
 }
 
 
-struct buffer_head * find_buffer (int dev, unsigned long block, unsigned long size)
+struct buffer_head *find_buffer(int dev, unsigned long block, unsigned long size)
 {		
     struct buffer_head * next;
 
@@ -346,8 +334,7 @@ static struct buffer_head * get_free_buffer (struct buffer_head **list,
 
 /* to_write == 0 when all blocks have to be flushed. Otherwise - write only
    buffers with b_count == 0 */
-static int sync_buffers (struct buffer_head **list, dev_t dev, int to_write)
-{
+static int sync_buffers (struct buffer_head **list, int dev, int to_write) {
     struct buffer_head * next;
     int written = 0;
 
@@ -388,10 +375,11 @@ restart:
 }
 
 
-void flush_buffers (dev_t dev)
+void flush_buffers (int dev)
 {
     if (dev == -1)
 	die ("flush_buffers: device is not specified");
+
     sync_buffers (&Buffer_list_head, dev, 0/*all*/);
     buffer_soft_limit = BUFFER_SOFT_LIMIT;
 }
@@ -455,9 +443,9 @@ void brelse (struct buffer_head * bh)
 {
     if (bh == 0)
 	return;
-    if (bh->b_count == 0) {
+    
+    if (bh->b_count == 0)
 	die ("brelse: can not free a free buffer %lu", bh->b_blocknr);
-    }
     
     /*checkmem (bh->b_data, get_mem_size (bh->b_data));*/
     
@@ -472,28 +460,25 @@ void bforget (struct buffer_head * bh)
 	brelse (bh);
 	remove_from_hash_queue (bh);
 	remove_from_buffer_list(&Buffer_list_head, bh);
-	if (bh->b_count == 0)
-	    put_buffer_list_head(&g_free_buffers, bh);
-	else
-	    put_buffer_list_head(&Buffer_list_head, bh);
+	put_buffer_list_head(&Buffer_list_head, bh);
     }
 }
 
 /* Returns 0 on success; 1 - end of file; 0 - OK. */
 static int f_read(struct buffer_head * bh)
 {
-    loff_t offset;
+    unsigned long long offset;
     ssize_t bytes;
 
     buffer_reads++ ;
 
-    offset = (loff_t)bh->b_size * (loff_t)bh->b_blocknr;
-    if (lseek64 (bh->b_dev, offset, SEEK_SET) < 0)
+    offset = (unsigned long long)bh->b_size * bh->b_blocknr;
+    if (lseek (bh->b_dev, offset, SEEK_SET) < 0)
 	return -1;
 
     bytes = read (bh->b_dev, bh->b_data, bh->b_size);
     
-    return bytes < 0 ? -1 : (bytes != bh->b_size ? 1 : 0);
+    return bytes < 0 ? -1 : (bytes != (ssize_t)bh->b_size ? 1 : 0);
 }
 
 
@@ -515,7 +500,7 @@ struct buffer_head * bread (int dev, unsigned long block, size_t size)
     ret = f_read(bh);
     
     if (ret > 0)
-	die ("%s: End of file, cannot read the block (%lu).\n", __FUNCTION__, bh->b_blocknr);
+	die ("%s: End of file, cannot read the block (%lu).\n", __FUNCTION__, block);
     else if (ret < 0) {
 	/* BAD BLOCK LIST SUPPORT
 	 * die ("%s: Cannot read a block # %lu. Specify list of badblocks\n",*/
@@ -523,10 +508,10 @@ struct buffer_head * bread (int dev, unsigned long block, size_t size)
 	if (errno == EIO) {
 	    check_hd_msg();
 	    die ("%s: Cannot read the block (%lu): (%s).\n", __FUNCTION__, 
-		bh->b_blocknr, strerror(errno));
+		block, strerror(errno));
 	} else	{
 	    fprintf (stderr, "%s: Cannot read the block (%lu): (%s).\n", __FUNCTION__, 
-		bh->b_blocknr, strerror(errno));
+		block, strerror(errno));
 	    return NULL;
 	}
     }
@@ -535,26 +520,6 @@ struct buffer_head * bread (int dev, unsigned long block, size_t size)
     return bh;
 }
 
-
-int valid_offset( int fd, loff_t offset)
-{
-    char ch;
-    loff_t res;
-
-    /*res = reiserfs_llseek (fd, offset, 0);*/
-    res = lseek64 (fd, offset, SEEK_SET);
-    if (res < 0)
-	return 0;
-
-    /* if (read (fd, &ch, 1) < 0) does not wirk on files */
-    if (read (fd, &ch, 1) < 1)
-	return 0;
-
-
-    return 1;
-}
-
-
 #define ROLLBACK_FILE_START_MAGIC       "_RollBackFileForReiserfsFSCK"
 
 static struct block_handler * rollback_blocks_array;
@@ -568,12 +533,12 @@ static int rollback_blocksize;
 
 void init_rollback_file (char * rollback_file, int *blocksize, FILE * log) {
     char * string;
-    struct stat64 buf;
+    struct stat buf;
     
     if (rollback_file == NULL)
         return;
         
-    stat64(rollback_file, &buf);
+    stat(rollback_file, &buf);
     
     s_rollback_file = fopen (rollback_file, "w+");    
     if (s_rollback_file == NULL) {
@@ -612,12 +577,12 @@ static void erase_rollback_file (char * rollback_file) {
 
 int open_rollback_file (char * rollback_file, FILE * log) {
     char string [28];
-    struct stat64 buf;
+    struct stat buf;
     
     if (rollback_file == NULL)
         return -1;
     
-    if (stat64(rollback_file, &buf)) {
+    if (stat(rollback_file, &buf)) {
 	fprintf (stderr, "Cannot stat rollback file (%s)\n", rollback_file);
 	return -1;
     }
@@ -679,17 +644,17 @@ void close_rollback_file () {
 }
 
 void do_fsck_rollback (int fd_device, int fd_journal_device, FILE * progress) {
-    loff_t offset;
+    long long int offset;
     
-    struct stat64 buf;
+    struct stat buf;
     int descriptor;
     ssize_t retval;
     int count_failed = 0;
     int count_rollbacked = 0;
     
-    dev_t b_dev;
-    dev_t n_dev = 0;
-    dev_t n_journal_dev = 0;
+    int b_dev;
+    int n_dev = 0;
+    int n_journal_dev = 0;
     unsigned long total, done = 0;
 
     if (fd_device == 0) {
@@ -698,14 +663,14 @@ void do_fsck_rollback (int fd_device, int fd_journal_device, FILE * progress) {
     }
         
     if (fd_journal_device) {
-        if (!fstat64 (fd_journal_device, &buf)) {
+        if (!fstat (fd_journal_device, &buf)) {
             n_journal_dev = buf.st_rdev;
         } else {
             fprintf(stderr, "rollback: specified journal device cannot be stated\n");
         }
     }
     
-    if (!fstat64 (fd_device, &buf)) {
+    if (!fstat (fd_device, &buf)) {
         n_dev = buf.st_rdev;
     } else {
         fprintf(stderr, "rollback: specified device cannot be stated, exit\n");
@@ -754,15 +719,16 @@ void do_fsck_rollback (int fd_device, int fd_journal_device, FILE * progress) {
             continue;
         }
         
-        if (lseek64 (descriptor, offset, SEEK_SET) == (loff_t)-1) {
+        if (lseek (descriptor, offset, SEEK_SET) == (loff_t)-1) {
             fprintf(stderr, "device cannot be lseeked, skip block\n");
             count_failed ++;
             continue;
         }
         
         if (write (descriptor, rollback_data, rollback_blocksize) == -1) {
-            fprintf (stderr, "rollback: write %d bytes returned error (block=%Ld, dev=%Ld): %s\n",
-		rollback_blocksize, (long long)offset/rollback_blocksize, (long long)b_dev, strerror (errno));
+            fprintf (stderr, "rollback: write %d bytes returned error "
+		"(block=%lld, dev=%d): %s\n", rollback_blocksize, 
+		offset/rollback_blocksize, b_dev, strerror (errno));
             count_failed ++;
         } else {
             count_rollbacked ++;
@@ -804,9 +770,8 @@ static void rollback__mark_block_saved (struct block_handler * rb_e) {
 /* for now - just make sure that bad blocks did not get here */
 int bwrite (struct buffer_head * bh)
 {
-    loff_t offset;
-    ssize_t bytes;
-    size_t size;
+    unsigned long long offset;
+    long long bytes, size;
 
     if (is_bad_block (bh->b_blocknr)) {
 	fprintf (stderr, "bwrite: bad block is going to be written: %lu\n",
@@ -825,62 +790,69 @@ int bwrite (struct buffer_head * bh)
     size = bh->b_size;
     offset = (loff_t)size * (loff_t)bh->b_blocknr;
 
-    if (lseek64 (bh->b_dev, offset, SEEK_SET) == (loff_t)-1){
-	fprintf (stderr, "bwrite: lseek to position %Ld (block=%lu, dev=%d): %s\n",
-		 (long long)offset, bh->b_blocknr, bh->b_dev, strerror (errno));
+    if (lseek (bh->b_dev, offset, SEEK_SET) == (loff_t)-1){
+	fprintf (stderr, "bwrite: lseek to position %llu (block=%lu, dev=%d): %s\n",
+	    offset, bh->b_blocknr, bh->b_dev, strerror(errno));
 	exit(8); /* File system errors left uncorrected */
     }
 
     if (s_rollback_file != NULL && bh->b_size == (unsigned long)rollback_blocksize) {
-        struct stat64 buf;
+        struct stat buf;
         __u32 position;
 	struct block_handler block_h;
         
         /*log previous content into the log*/
-        if (!fstat64 (bh->b_dev, &buf)) {
+        if (!fstat (bh->b_dev, &buf)) {
 	    block_h.blocknr = bh->b_blocknr;
 	    block_h.device = buf.st_rdev;
-	    if (reiserfs_bin_search (&block_h, rollback_blocks_array, rollback_blocks_number,
-		sizeof (block_h), &position, blockdev_list_compare) != POSITION_FOUND) {
+	    if (reiserfs_bin_search(&block_h, rollback_blocks_array, 
+				    rollback_blocks_number, sizeof (block_h), 
+				    &position, blockdev_list_compare)
+		!= POSITION_FOUND)
+	    {
                 /*read initial data from the disk*/
-                if (read (bh->b_dev, rollback_data, bh->b_size) == (ssize_t)bh->b_size) {
-                    fwrite (&buf.st_rdev, sizeof (buf.st_rdev), 1, s_rollback_file);
-                    fwrite (&offset, sizeof (offset), 1, s_rollback_file);
-                    fwrite (rollback_data, rollback_blocksize, 1, s_rollback_file);
+                if (read(bh->b_dev, rollback_data, bh->b_size) == (long long)bh->b_size) {
+                    fwrite(&buf.st_rdev, sizeof (buf.st_rdev), 1, s_rollback_file);
+                    fwrite(&offset, sizeof (offset), 1, s_rollback_file);
+                    fwrite(rollback_data, rollback_blocksize, 1, s_rollback_file);
                     fflush(s_rollback_file);
-                    blocklist__insert_in_position(&block_h, (void **)&rollback_blocks_array, 
+                    blocklist__insert_in_position(&block_h, (void *)(&rollback_blocks_array),
 			&rollback_blocks_number, sizeof(block_h), &position);
+		    
                     /*if you want to know what gets saved, uncomment it*/
 /*                    if (log_file != 0 && log_file != stdout) {
-                        fprintf (log_file, "rollback: block %lu of device %Lu was backed up\n", 
-                                bh->b_blocknr, buf.st_rdev);
+                        fprintf (log_file, "rollback: block %lu of device %Lu was "
+			    "backed up\n", bh->b_blocknr, buf.st_rdev);
                     }
 */
                     
                 } else {
-                    fprintf (stderr, "bwrite: read (block=%lu, dev=%d): %s\n", bh->b_blocknr,
-                    		bh->b_dev, strerror (errno));
+                    fprintf (stderr, "bwrite: read (block=%lu, dev=%d): %s\n", 
+			bh->b_blocknr, bh->b_dev, strerror (errno));
                     exit(8);
                 }
-                if (lseek64 (bh->b_dev, offset, SEEK_SET) == (loff_t)-1) {
-                    fprintf (stderr, "bwrite: lseek to position %Ld (block=%lu, dev=%d): %s\n",
-        		 (long long)offset, bh->b_blocknr, bh->b_dev, strerror (errno));
+		
+                if (lseek (bh->b_dev, offset, SEEK_SET) == (loff_t)-1) {
+                    fprintf (stderr, "bwrite: lseek to position %llu (block=%lu, "
+			"dev=%d): %s\n", offset, bh->b_blocknr, bh->b_dev, 
+			strerror(errno));
                     exit(8);
                 }
             }
         } else {
-            fprintf (stderr, "bwrite: fstat of (%d) returned -1: %s\n", bh->b_dev, 
-		strerror(errno));
+            fprintf (stderr, "bwrite: fstat of (%d) returned -1: %s\n", 
+		bh->b_dev, strerror(errno));
         }
     } else if (s_rollback_file != NULL) {
-	fprintf (stderr, "rollback: block (%lu) has the size different from the fs uses, "
-	    "block skipped\n",	bh->b_blocknr);
+	fprintf (stderr, "rollback: block (%lu) has the size different from "
+	    "the fs uses, block skipped\n", bh->b_blocknr);
     }
     
-    bytes = write (bh->b_dev, bh->b_data, size);
-    if (bytes != (ssize_t)size) {
-	fprintf (stderr, "bwrite: write %ld bytes returned %ld (block=%ld, dev=%d): %s\n",
-		(long)size, (long)bytes, bh->b_blocknr, bh->b_dev, strerror (errno));
+    bytes = write(bh->b_dev, bh->b_data, size);
+    if (bytes != size) {
+	fprintf (stderr, "bwrite: write %lld bytes returned %lld (block=%ld, "
+	    "dev=%d): %s\n", size, bytes, bh->b_blocknr, bh->b_dev, 
+	    strerror(errno));
 	exit(8);
     }
 
@@ -902,12 +874,13 @@ static int _check_and_free_buffer_list(struct buffer_head *list) {
 
     for(;;) {
 	if (next->b_count != 0)
-	    fprintf (stderr, "check_and_free_buffer_mem: not free buffer (%x, %ld, %ld, %d)\n",
-		     next->b_dev, next->b_blocknr, next->b_size, next->b_count);
+	    fprintf (stderr, "check_and_free_buffer_mem: not free buffer "
+		"(%d, %ld, %ld, %d)\n", next->b_dev, next->b_blocknr, 
+		next->b_size, next->b_count);
 
 	if (buffer_dirty (next) && buffer_uptodate (next))
-	    fprintf (stderr, "check_and_free_buffer_mem: dirty buffer (%x %lu) found\n",
-		     next->b_dev, next->b_blocknr);
+	    fprintf (stderr, "check_and_free_buffer_mem: dirty buffer "
+		"(%d %lu) found\n", next->b_dev, next->b_blocknr);
 
 	freemem (next->b_data);
 	count++;
@@ -918,7 +891,7 @@ static int _check_and_free_buffer_list(struct buffer_head *list) {
     return count;
 }
 
-void check_and_free_buffer_mem (void)
+static void check_and_free_buffer_mem (void)
 {
     int count = 0;
     struct buffer_head * next ;
@@ -950,7 +923,7 @@ void free_buffers (void)
 }
 
 
-static void _invalidate_buffer_list(struct buffer_head *list, dev_t dev)
+static void _invalidate_buffer_list(struct buffer_head *list, int dev)
 {
     struct buffer_head * next;
 
@@ -974,21 +947,8 @@ static void _invalidate_buffer_list(struct buffer_head *list, dev_t dev)
 }
 
 /* forget all buffers of the given device */
-void invalidate_buffers (dev_t dev)
+void invalidate_buffers (int dev)
 {
     _invalidate_buffer_list(Buffer_list_head, dev) ;
     _invalidate_buffer_list(g_free_buffers, dev) ;
 }
-
-
-int user_confirmed (FILE * fp, char * q, char * yes)
-{
-    char * answer = 0;
-    size_t n = 0;
-
-    fprintf (fp, "%s", q);
-    if (getline (&answer, &n, stdin) != (ssize_t)strlen (yes) || strcmp (yes, answer))
-	return 0;
-
-    return 1;
-}
