@@ -1,7 +1,8 @@
 /* 
- * Copyright 2000-2003 by Hans Reiser, licensing governed by reiserfs/README
+ * Copyright 2000-2004 by Hans Reiser, licensing governed by 
+ * reiserfsprogs/README
  */
-  
+
 /*
  * 2000/10/26 - Initial version.
  */
@@ -47,15 +48,28 @@ int reiserfs_expand_bitmap (reiserfs_bitmap_t * bm, unsigned int bit_count)
     bm->bm_map = new_map;
     bm->bm_byte_size = byte_count;
     bm->bm_bit_size = bit_count;
+
+    bm->bm_dirty = 1;
+
     return 0;
 }
 
 void reiserfs_shrink_bitmap (reiserfs_bitmap_t * bm, unsigned int bit_count)
 {
+    unsigned long i;
+    
     assert (bm->bm_bit_size >= bit_count);
 
     bm->bm_byte_size = (bit_count + 7) / 8;
     bm->bm_bit_size = bit_count;
+    bm->bm_set_bits = 0;
+    
+    bm->bm_dirty = 1;
+    
+    for (i = 0; i < bm->bm_bit_size; i++) {
+	if (reiserfs_bitmap_test_bit(bm, i))
+	    bm->bm_set_bits++;
+    }
 }
 
 /* bitmap destructor */
@@ -80,8 +94,8 @@ void reiserfs_bitmap_copy (reiserfs_bitmap_t * to, reiserfs_bitmap_t * from)
 
 int reiserfs_bitmap_compare (reiserfs_bitmap_t * bm1, reiserfs_bitmap_t * bm2)
 {
-    int bytes, bits;
-    long unsigned i, diff;
+    unsigned long i, diff;
+    unsigned long int bytes, bits;
 
     assert (bm1->bm_byte_size == bm2->bm_byte_size &&
 	    bm1->bm_bit_size == bm2->bm_bit_size);
@@ -119,9 +133,10 @@ int reiserfs_bitmap_compare (reiserfs_bitmap_t * bm1, reiserfs_bitmap_t * bm2)
     Y - `from` bitmap. 
     Save result in the `to` bitmap.
 */
-void reiserfs_bitmap_disjunction (reiserfs_bitmap_t * to, reiserfs_bitmap_t * from)
+void reiserfs_bitmap_disjunction (reiserfs_bitmap_t * to, 
+    reiserfs_bitmap_t * from) 
 {
-    int i;
+    unsigned int i;
 
     assert (to->bm_byte_size == from->bm_byte_size &&
 	    to->bm_bit_size == from->bm_bit_size);
@@ -142,7 +157,7 @@ void reiserfs_bitmap_disjunction (reiserfs_bitmap_t * to, reiserfs_bitmap_t * fr
     Save result in the `base` bitmap.
 */
 void reiserfs_bitmap_delta (reiserfs_bitmap_t * base, reiserfs_bitmap_t * exclude) {
-   int i;
+   unsigned int i;
 
     assert (base->bm_byte_size == exclude->bm_byte_size &&
 	    base->bm_bit_size == exclude->bm_bit_size);
@@ -187,30 +202,28 @@ int reiserfs_bitmap_test_bit (reiserfs_bitmap_t * bm, unsigned int bit_number)
 }
 
 
-int reiserfs_bitmap_zeros (reiserfs_bitmap_t * bm)
-{
+unsigned int reiserfs_bitmap_zeros (reiserfs_bitmap_t * bm) {
     return bm->bm_bit_size - bm->bm_set_bits;
 }
 
 
-int reiserfs_bitmap_ones (reiserfs_bitmap_t * bm)
-{
+unsigned int reiserfs_bitmap_ones (reiserfs_bitmap_t * bm) {
     return bm->bm_set_bits;
 }
 
 
-int reiserfs_bitmap_find_zero_bit (reiserfs_bitmap_t * bm, unsigned long * start)
+int reiserfs_bitmap_find_zero_bit (reiserfs_bitmap_t * bm, unsigned long * first)
 {
-    unsigned int  bit_nr = *start;
-    assert(*start < bm->bm_bit_size);
+    unsigned long bit_nr = *first;
+    assert(*first < bm->bm_bit_size);
 
-    bit_nr = misc_find_next_zero_bit(bm->bm_map, bm->bm_bit_size, *start);
+    bit_nr = misc_find_next_zero_bit(bm->bm_map, bm->bm_bit_size, *first);
 
     if (bit_nr >= bm->bm_bit_size) { /* search failed */	
 	return 1;
     }
 
-    *start = bit_nr;
+    *first = bit_nr;
     return 0;
 }
 
@@ -218,13 +231,13 @@ int reiserfs_bitmap_find_zero_bit (reiserfs_bitmap_t * bm, unsigned long * start
 /* read every bitmap block and copy their content into bitmap 'bm' */
 static int reiserfs_fetch_ondisk_bitmap (reiserfs_bitmap_t * bm, reiserfs_filsys_t * fs)
 {
-    unsigned long to_copy;
-    int copied;
-    int i, ret = 0;
-    char * p;
-    int last_byte_unused_bits;
-    unsigned long block;
+    unsigned int last_byte_unused_bits;
+    unsigned long block, to_copy;
     struct buffer_head * bh;
+    unsigned int i;
+    int copied;
+    int ret = 0;
+    char * p;
 
     to_copy = (get_sb_block_count (fs->fs_ondisk_sb) + 7) / 8;
 
@@ -272,6 +285,7 @@ static int reiserfs_fetch_ondisk_bitmap (reiserfs_bitmap_t * bm, reiserfs_filsys
     /* on disk bitmap has bits out of SB_BLOCK_COUNT set to 1, where as
        reiserfs_bitmap_t has those bits set to 0 */
     last_byte_unused_bits = bm->bm_byte_size * 8 - bm->bm_bit_size;
+
     for (i = 0; i < last_byte_unused_bits; i ++) {
 	if (misc_test_bit (bm->bm_bit_size + i, bm->bm_map) == 0)
 	    ret = 1;
@@ -295,14 +309,10 @@ static int reiserfs_fetch_ondisk_bitmap (reiserfs_bitmap_t * bm, reiserfs_filsys
    changed and return 1. Otherwise - return 0 */
 int reiserfs_flush_to_ondisk_bitmap (reiserfs_bitmap_t * bm, reiserfs_filsys_t * fs)
 {
-    unsigned long to_copy;
-    int copied;
-    int i;
-    char * p;
-    int last_byte_unused_bits;
-    unsigned long block;
+    unsigned int last_byte_unused_bits, i;
+    unsigned long to_copy, copied, block;
     struct buffer_head * bh;
-
+    char * p;
 
     /* make sure that the device is big enough */
     bh = bread (fs->fs_dev, bm->bm_bit_size - 1, fs->fs_blocksize);
@@ -343,6 +353,7 @@ int reiserfs_flush_to_ondisk_bitmap (reiserfs_bitmap_t * bm, reiserfs_filsys_t *
 	if (copied == to_copy) {
 	    /* set unused bits of last byte of a bitmap to 1 */
 	    last_byte_unused_bits = bm->bm_byte_size * 8 - bm->bm_bit_size;
+
 	    for (i = 0; i < last_byte_unused_bits; i ++)
 		misc_set_bit ((bm->bm_bit_size % (fs->fs_blocksize * 8)) + i, bh->b_data);
 	}
@@ -415,7 +426,7 @@ void reiserfs_bitmap_save (FILE * fp, reiserfs_bitmap_t * bm)
     __u32 v;
     int zeros;
     int count;
-    int i;
+    unsigned int i;
     int extents;
     long position;
 
@@ -647,7 +658,7 @@ reiserfs_bitmap_t * reiserfs_bitmap_load (FILE * fp)
 
 void reiserfs_bitmap_invert (reiserfs_bitmap_t * bm)
 {
-    int i;
+    unsigned int i;
 
     /*reiserfs_warning (stderr, "Bitmap inverting..");fflush (stderr);*/
     for (i = 0; i < bm->bm_bit_size; i ++) {
