@@ -3,15 +3,9 @@
  * reiserfsprogs/README
  */
 
-#include "includes.h"
+#define _GNU_SOURCE
 
-/* this is provided for anybody who wants to deal with journal */
-int replay_one_transaction (reiserfs_filsys_t *, reiserfs_trans_t *);
-void for_each_transaction (reiserfs_filsys_t *, action_on_trans_t);
-void for_each_block (reiserfs_filsys_t *, reiserfs_trans_t *, action_on_block_t);
-int get_boundary_transactions (reiserfs_filsys_t *, reiserfs_trans_t *,
-			       reiserfs_trans_t *);
-int next_transaction (reiserfs_filsys_t *, reiserfs_trans_t *, reiserfs_trans_t);
+#include "includes.h"
 
 /* compares description block with commit block. returns 0 if they differ, 1
    if they match */
@@ -491,7 +485,11 @@ int reiserfs_open_journal (reiserfs_filsys_t * fs, char * j_filename, int flags)
 	}
     }
 
-    fs->fs_journal_dev = open (j_filename, flags | O_LARGEFILE);
+    fs->fs_journal_dev = open (j_filename, flags 
+#if defined(O_LARGEFILE)
+			       | O_LARGEFILE
+#endif
+			       );
     if (fs->fs_journal_dev == -1) 
         return -1;
     
@@ -618,7 +616,11 @@ int reiserfs_create_journal(
 */
     }
 
-    fs->fs_journal_dev = open (j_device, O_RDWR | O_LARGEFILE);
+    fs->fs_journal_dev = open (j_device, O_RDWR 
+#if defined(O_LARGEFILE)
+			       | O_LARGEFILE
+#endif
+			       );
     if (fs->fs_journal_dev == -1) {
 	reiserfs_warning (stderr, "reiserfs_create_journal: could not open "
 	    "%s: %s\n", j_device, strerror(errno));
@@ -685,7 +687,11 @@ void reiserfs_reopen_journal (reiserfs_filsys_t * fs, int flag)
     if (close (fs->fs_journal_dev))
 	die ("reiserfs_reopen_journal: closed failed: %s", strerror(errno));
 
-    fs->fs_journal_dev = open (fs->fs_j_file_name, flag | O_LARGEFILE);
+    fs->fs_journal_dev = open (fs->fs_j_file_name, flag 
+#if defined(O_LARGEFILE)
+			       | O_LARGEFILE
+#endif
+			       );
     if (fs->fs_journal_dev == -1)
 	die ("reiserfs_reopen_journal: could not reopen journal device");
 
@@ -786,8 +792,8 @@ int replay_journal (reiserfs_filsys_t * fs)
     
     while (ret == TRANS_FOUND) {
 	/* If not the next transaction to be replayed, break out here. */
-	if (cur.mount_id != control.mount_id || 
-	    cur.trans_id != control.trans_id + 1 && control.trans_id)
+	if ((cur.mount_id != control.mount_id) || 
+	    (cur.trans_id != control.trans_id + 1 && control.trans_id))
 	    break;
 	
 	if (!transaction_check_content(fs, &cur)) {
