@@ -3,8 +3,15 @@
  * reiserfsprogs/README
  */
 
+#define _GNU_SOURCE
+
 #include "tune.h"
 
+#include <getopt.h>
+#include <stdarg.h>
+#include <string.h>
+#include <errno.h>
+
 char *program_name;
 
 static void message( const char * fmt, ... ) 
@@ -467,7 +474,7 @@ int main (int argc, char **argv)
        --no-journal-available has been specified by user */
 
     /* make sure filesystem is not mounted */
-    if (misc_device_mounted(fs->fs_file_name, 0)) {
+    if (misc_device_mounted(fs->fs_file_name) > 0) {
 	/* fixme: it can not be mounted, btw */
         message ("Reiserfstune is not allowed to be run on mounted filesystem.");
 	reiserfs_close (fs);
@@ -514,7 +521,11 @@ int main (int argc, char **argv)
     if (get_jp_journal_magic(sb_jp(fs->fs_ondisk_sb)) != NEED_TUNE && 
 	!(Options & OPT_SKIP_J)) 
     {
-	if (reiserfs_open_journal (fs, jdevice_name, O_RDWR | O_LARGEFILE)) {
+	if (reiserfs_open_journal (fs, jdevice_name, O_RDWR 
+#if defined(O_LARGEFILE)
+				   | O_LARGEFILE
+#endif
+				   )) {
 	    message ("Failed to open the journal device (%s).", jdevice_name);
 	    return 1;
 	}
